import { EditorView } from '@codemirror/view';
import type { KeyBinding } from '@codemirror/view';
import type { Text } from '@codemirror/state';
import type { Diagnostic } from '@codemirror/lint';
export type { MwConfig } from './mediawiki';
export type LintSource = (doc: Text) => Diagnostic[] | Promise<Diagnostic[]>;
export declare const CDN = "https://testingcf.jsdelivr.net";
export declare class CodeMirror6 {
    #private;
    get textarea(): HTMLTextAreaElement;
    get view(): EditorView;
    get lang(): string;
    get visible(): boolean;
    /**
     * @param textarea 文本框
     * @param lang 语言
     * @param config 语言设置
     */
    constructor(textarea: HTMLTextAreaElement, lang?: string, config?: unknown);
    /**
     * 设置语言
     * @param lang 语言
     * @param config 语言设置
     */
    setLanguage(lang?: string, config?: unknown): void;
    /**
     * 开始语法检查
     * @param lintSource 语法检查函数
     */
    lint(lintSource?: LintSource): void;
    /** 立即更新语法检查 */
    update(): void;
    /**
     * 添加扩展
     * @param names 扩展名
     */
    prefer(names: string[] | Record<string, boolean>): void;
    /**
     * 设置缩进
     * @param indent 缩进字符串
     */
    setIndent(indent: string): void;
    /** 获取默认linter */
    getLinter(opt?: Record<string, unknown>): Promise<LintSource | undefined>;
    /**
     * 重设编辑器内容
     * @param content 新内容
     */
    setContent(content: string): void;
    /**
     * 在编辑器和文本框之间切换
     * @param show 是否显示编辑器
     */
    toggle(show?: boolean): void;
    /**
     * 添加额外快捷键
     * @param keys 快捷键
     */
    extraKeys(keys: KeyBinding[]): void;
    /**
     * 替换选中内容
     * @param view EditorView
     * @param func 替换函数
     */
    static replaceSelections(view: EditorView, func: (str: string) => string): void;
}
